#ifndef __CLD_AUDIO_1_0_LIB__
#define __CLD_AUDIO_1_0_LIB__
/*==============================================================================
    FILE:           cld_bf70x_audio_1_0_lib.h

    DESCRIPTION:    CLD Audio v1.0 peripheral Library.

    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/
#ifndef CLD_NULL
#define CLD_NULL    0
#endif

#if defined(_LANGUAGE_C) || defined(__cplusplus)

#ifdef __cplusplus
extern "C"
{
#endif


/* CLD Timer numbers used to select which of the BF707 timers is used by the
   CLD library. */
typedef enum
{
    CLD_TIMER_0 = 1,
    CLD_TIMER_1,
    CLD_TIMER_2,
    CLD_TIMER_3,
    CLD_TIMER_4,
    CLD_TIMER_5,
    CLD_TIMER_6,
    CLD_TIMER_7,
    CLD_TIMER_MAX
} CLD_Timer_Num;

typedef unsigned long CLD_Time;

/* CLD UART numbers used to select which of the BF707 UARTs is used by the
   CLD library. */
typedef enum
{
    CLD_UART_0 = 1,
    CLD_UART_1 = 2,
    CLD_UART_DISABLE,
} CLD_Uart_Num;

/* CLD Library return codes. */
typedef enum
{
    CLD_SUCCESS = 0,
    CLD_FAIL = 1,
    CLD_ONGOING = 2
} CLD_RV;

typedef enum            /* Simple boolean definitions */
{
    CLD_FALSE = 0,
    CLD_TRUE  = 1,
} CLD_Boolean;

/* CLD Library USB Events */
typedef enum
{
    CLD_USB_CABLE_CONNECTED = 0,
    CLD_USB_CABLE_DISCONNECTED,
    CLD_USB_ENUMERATED_CONFIGURED,
    CLD_USB_UN_CONFIGURED,
    CLD_USB_BUS_RESET,
} CLD_USB_Event;

/* Value returned by the User code to select how the requested transfer should
   be handled. */
typedef enum
{
    CLD_USB_TRANSFER_ACCEPT = 0,                        /* Accept the requested transfer using
                                                           the supplied CLD_USB_Transfer_Params. */
    CLD_USB_TRANSFER_PAUSE,                             /* Pause the current transfer. The USB
                                                           endpoint will be Nak'ed until the
                                                           appropriate 'resume' function is
                                                           called. This is used to throttle
                                                           the USB host. */
    CLD_USB_TRANSFER_DISCARD,                           /* Discard the transfer. For OUT requests
                                                           the received data is discarded. For IN
                                                           requests the device will return a
                                                           zero length packet. */
    CLD_USB_TRANSFER_STALL,                             /* Stall the transfer request. */
} CLD_USB_Transfer_Request_Return_Type;

/* Value returned by the User code to notify the USB library if the received data
   was valid. */
typedef enum
{
    CLD_USB_DATA_GOOD = 0,                              /* Received data is good.
                                                           For Control OUT requests this
                                                           allows the Status Stage to
                                                           complete. */
    CLD_USB_DATA_BAD_STALL,                             /* Received data is bad so stall the endpoint
                                                           For Control OUT requests this
                                                           stalls the Status Stage, for
                                                           other endpoint types the
                                                           next OUT packet will be stalled. */
} CLD_USB_Data_Received_Return_Type;

/* Value returned by the USB library to notify the User code if the requested data
   can be transmitted. */
typedef enum
{
    CLD_USB_TRANSMIT_SUCCESSFUL = 0,
    CLD_USB_TRANSMIT_FAILED,
} CLD_USB_Data_Transmit_Return_Type;

/* USB transfer request parameters used to tell the USB library how to handle the
   current transfer. */
typedef struct
{
    unsigned long num_bytes;                            /* The number of bytes to
                                                           transmit or receive depending
                                                           on the transfer direction (IN/OUT) */
    unsigned char * p_data_buffer;                      /* Pointer to the data to transmit
                                                           (IN Transfer) or a pointer to
                                                           the buffer to store the received
                                                           data (OUT transfer). */

    /* The following function pointers are used by the USB Library to notify the
       User code when certain events occur.  These function pointers can be set
       to NULL if the User does not want to be notified. */
    union
    {                                                   /* Function called when the data
                                                           has been received. */
        CLD_USB_Data_Received_Return_Type (*fp_usb_out_transfer_complete)(void);
        void (*fp_usb_in_transfer_complete) (void);     /* Function called when the requested
                                                           data has been transmitted. */
    }callback;
    void (*fp_transfer_aborted_callback) (void);        /* Function called if the requested
                                                           transfer is aborted. */
    CLD_Time transfer_timeout_ms;                       /* Transfer Timeout in milliseconds
                                                           (0 - Disables the timeout) */
} CLD_USB_Transfer_Params;

#pragma pack (1)

/* USB Audio v1.0 Audio Stream Isochronous Data Endpoint Descriptor
   (refer to: USB Device Class Definition of Audio Devices v 1.0 section 4.6.2.1) */
typedef struct
{
    unsigned char  b_length;
    unsigned char  b_descriptor_type;                   /* Descriptor type = 0x25 (Class Specific Endpoint) */
    unsigned char  b_descriptor_subtype;                /* Descriptor subtype = 0x01 (Endpoint General) */
    unsigned char  bm_attributes;                       /* Identifies the attributes supported by this endpoint
                                                           Bit 0: Sampling Frequency
                                                           Bit 1: Pitch
                                                           Bit 7: Max Packets Only */
    unsigned char  b_lock_delay_units;                  /* Specifies the units used for w_lock_delay
                                                           0 = undefined
                                                           1 = Milliseconds
                                                           2 = Decoded PCM samples */
    unsigned short w_lock_delay;                        /* Specifies the time it takes the endpoint
                                                           to reliably lock its internal clock recovery */
} CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor;


#pragma pack ()
/* Parameters used to configure the USB Audio Stream Isochronous Interface. */
typedef struct
{
    /* Parameters used for the Isochronous Endpoint Descriptor */
    unsigned char endpoint_number;                      /* Isochronous endpoint number */
    unsigned short max_packet_size_full_speed;          /* Isochronous endpoint full-speed max packet size */
    unsigned short max_packet_size_high_speed;          /* Isochronous endpoint high-speed max packet size */
    unsigned char b_interval_full_speed;                /* Isochronous endpoint full-speed bInterval */
    unsigned char b_interval_high_speed;                /* Isochronous endpoint high-speed bInterval */
    unsigned char synchronization_type;                 /* Isochronous endpoint synchronization type
                                                            1 = Asynchronous
                                                            2 = Adaptive
                                                            3 = Synchronous */
    /* Parameters used for Audio Stream Interface Descriptor */
    unsigned char b_terminal_link;                      /* The Terminal ID connected to the Isochronous endpoint */
    unsigned char b_delay;                              /* Delay (in frames) introduced by the data path */
    unsigned short w_format_tag;                        /* Audio Data Format */

    unsigned char * p_format_type_descriptor;           /* Pointer to the Interface's format type descriptor */
                                                        /* Pointer to the Audio Stream endpoint data descriptor */
    CLD_BF70x_Audio_1_0_Lib_Audio_Stream_Data_Endpoint_Descriptor * p_audio_stream_endpoint_data_descriptor;
} CLD_BF70x_Audio_1_0_Stream_Interface_Params;

/* USB Audio v1.0 Set and Get Control endpoint bRequest values */
typedef enum
{
    CLD_GET_CURRENT     = 0x81,
    CLD_GET_MIN         = 0x82,
    CLD_GET_MAX         = 0x83,
    CLD_GET_RESOLUTION  = 0x84,
    CLD_GET_MEMORY      = 0x85,
    CLD_GET_STATUS      = 0xff,

    CLD_SET_CURRENT     = 0x01,
    CLD_SET_MIN         = 0x02,
    CLD_SET_MAX         = 0x03,
    CLD_SET_RESOLUTION  = 0x04,
    CLD_SET_MEMORY      = 0x05,
} CLD_BD70x_Audio_1_0_Requests;

/* Parameters passed to the fp_audio_set_req_cmd and fp_audio_get_req_cmd functions used to
   identify the active command. */
typedef struct
{
    CLD_BD70x_Audio_1_0_Requests    req;                /* bRequest value of the active command */
    CLD_Boolean                     recipient_is_interface; /* CLD_TRUE - The request was sent to the interface specified in interface_or_endpoint_num
                                                               CLD_FALSE - The request was sent to the streaming endpoint specified in interface_or_endpoint_num */
    unsigned char                   entity_id;          /* Target entity ID (Unit ID, Terminal ID, etc) */
                                                        /* Target interface or endpoint number depending on
                                                           the entity. */
    unsigned char                   interface_or_endpoint_num;
                                                        /* Setup Packet wValue */
    unsigned short                  setup_packet_wValue;
} CLD_BF70x_Audio_1_0_Cmd_Req_Parameters;

/* Table Entry used to specify user defined USB strings. For example these strings could be used along with
   the Unit and Terminal descriptor to include a descriptive string for an entity. */
typedef struct
{
    unsigned char string_index;                         /* String Descriptor index. */
    unsigned char * p_string;                           /* Null terminated string */
} CLD_BF70x_Audio_1_0_Lib_User_String_Descriptors;


/* CLD USB Audio v1.0 Library initialization parameters */
typedef struct
{
    CLD_Timer_Num timer_num;                            /* Timer used by the CLD Library */
    CLD_Uart_Num uart_num;                              /* UART used by the CLD Library.
                                                            If the uart_num is set to
                                                            CLD_UART_DISABLE the CLD library
                                                            will not use a UART */
    unsigned long uart_baud;                            /* CLD Library CONSOLE print UART
                                                           baudrate. */
    unsigned long sclk0;                                /* Blackfin SCLK0 frequency */
    void (*fp_console_rx_byte) (unsigned char byte);    /* Function called when a byte
                                                           is received by the CONSOLE
                                                           UART. */

    unsigned short vendor_id;                           /* USB Vendor ID */
    unsigned short product_id;                          /* USB Product ID */

    /* Pointer to the beginning of the Unit and Terminal Descriptors (these descriptors need to be packed). */
    unsigned char * p_unit_and_terminal_descriptors;
    /* The total length of the above Unit and Terminal Descriptors. */
    unsigned short  unit_and_terminal_descriptors_length;

    /* Pointer to the interface parameters for the Audio Stream RX Isochronous OUT interface.
       Set to CLD_NULL if not required.*/
    CLD_BF70x_Audio_1_0_Stream_Interface_Params * p_audio_streaming_rx_interface_params;

    /* Pointer to the interface parameters for the Audio Stream TX Isochronous IN interface.
       Set to CLD_NULL if not required.*/
    CLD_BF70x_Audio_1_0_Stream_Interface_Params * p_audio_streaming_tx_interface_params;

    /* This function is called when the data is received on the Isochronous OUT endpoint.
        If this function returns an error it causes the next Isochronous OUT packet to be stalled. */
    CLD_USB_Transfer_Request_Return_Type (*fp_audio_stream_data_received) (CLD_USB_Transfer_Params * p_transfer_data);

    /* Function called when an USB Audio 1.0 Set Request is received. The User code should set
       p_transfer_data->p_data_buffer to address where to store the data for the command
       described by p_req_params. */
    CLD_USB_Transfer_Request_Return_Type (*fp_audio_set_req_cmd) (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data);

    /* Function called when an USB Audio 1.0 Get Request is received. The User code should
       set p_transfer_data->p_data_buffer and p_transfer_data->num_bytes to describe the
       data to return to the Host */
    CLD_USB_Transfer_Request_Return_Type (*fp_audio_get_req_cmd) (CLD_BF70x_Audio_1_0_Cmd_Req_Parameters * p_req_params, CLD_USB_Transfer_Params * p_transfer_data);

    /* Function called when the Isochronous OUT endpoint interface is enabled/disabled by
       the USB Host using the Set Interface Alternate Setting parameter. */
    void (*fp_audio_streaming_rx_endpoint_enabled) (CLD_Boolean enabled);
    /* Function called when the Isochronous IN endpoint interface is enabled/disabled by
       the USB Host using the Set Interface Alternate Setting parameter. */
    void (*fp_audio_streaming_tx_endpoint_enabled) (CLD_Boolean enabled);

    unsigned char usb_bus_max_power;                    /* USB Configuration Descriptor
                                                           bMaxPower value (0 = self powered).
                                                           See USB 2.0 Section 9.6.3. */

    unsigned short device_descriptor_bcdDevice;         /* USB Device Descriptor bcdDevice
                                                           value. See USB 2.0 section 9.6.1. */
    /* USB string descriptors - Set to CLD_NULL if not required */
    const char * p_usb_string_manufacturer;
    const char * p_usb_string_product;
    const char * p_usb_string_serial_number;
    const char * p_usb_string_configuration;
    const char * p_usb_string_audio_control_interface;
    const char * p_usb_string_audio_streaming_out_interface;
    const char * p_usb_string_audio_streaming_in_interface;

                                                        /* Number of entries in the
                                                           p_user_string_descriptor_table array
                                                           (Set to 0 if not used) */
    unsigned char user_string_descriptor_table_num_entries;
                                                        /* Array of USB specified String Descriptors.
                                                           (Set to CLD_NULL if not used.) */
    CLD_BF70x_Audio_1_0_Lib_User_String_Descriptors * p_user_string_descriptor_table;

    unsigned short usb_string_language_id;              /* USB String Descriptor Language ID code
                                                           (0x0409 = English US). */

    /* Function called when one of the following USB events occurs
        CLD_USB_CABLE_CONNECTED      - USB cable connected
        CLD_USB_CABLE_DISCONNECTED   - USB cable disconnected
        CLD_USB_ENUMERATED_CONFIGURED- USB Enumerated (USB Configuration set to non-zero value)
        CLD_USB_UN_CONFIGURED        - USB Configuration set to 0.
        CLD_USB_BUS_RESET            - USB Bus Reset received. */
    void (*fp_cld_usb_event_callback) (CLD_USB_Event event);
} CLD_BF70x_Audio_1_0_Lib_Init_Params;


/* Timer functions used to measure time in milliseconds. */
extern CLD_Time cld_time_get(void);
extern CLD_Time cld_time_passed_ms(CLD_Time time);


/* Initializes the CLD Audio v1.0 library. This function needs to be called until
   CLD_SUCCESS or CLD_FAIL is returned.  If CLD_FAIL is returned there was a
   problem initializing the library, and any relevant error messages will be
   output using the cld_console UART. */
extern CLD_RV cld_bf70x_audio_1_0_lib_init (CLD_BF70x_Audio_1_0_Lib_Init_Params * cld_bf70x_audio_1_0_lib_params);
/* CLD Audio v1.0 library mainline function. The CLD library mainline function is
   required and should be called in each iteration of the main program loop. */
extern void cld_bf70x_audio_1_0_lib_main (void);

/* Function used to transmit audio data using the Isochronous IN endpoint.
   If the Isochronous IN endpoint is disabled this function will return
   CLD_USB_TRANSMIT_FAILED. */
extern CLD_USB_Data_Transmit_Return_Type cld_bf70x_audio_1_0_lib_transmit_audio_data (CLD_USB_Transfer_Params * p_transfer_data);

/* Function used to resume a paused Isochronous OUT transfer. When this function is called
   the fp_audio_stream_data_received function will be called with the CLD_USB_Transfer_Params
   of the paused transfer.  The fp_audio_stream_data_received function can then
   accept, discard or stall the previously paused transfer. */
extern void cld_bf70x_audio_1_0_lib_resume_paused_audio_data_transfer (void);

/* Function used to resume a paused control transfer. When this function is called
   the fp_audio_set_req_cmd or fp_audio_get_req_cmd function
   will be called depending on which request was paused.  The function can then
   accept, discard or stall the previously paused transfer. */
extern void cld_bf70x_audio_1_0_lib_resume_paused_control_transfer (void);

/* Connects to the USB Host. */
extern void cld_lib_usb_connect (void);
/* Disconnects from the USB Host. */
extern void cld_lib_usb_disconnect (void);

typedef enum
{
    CLD_CONSOLE_BLACK  = 0,
    CLD_CONSOLE_RED       ,
    CLD_CONSOLE_GREEN     ,
    CLD_CONSOLE_YELLOW    ,
    CLD_CONSOLE_BLUE      ,
    CLD_CONSOLE_PURPLE    ,
    CLD_CONSOLE_CYAN      ,
    CLD_CONSOLE_WHITE     ,
    CLD_CONSOLE_NUM_COLORS,
} CLD_CONSOLE_COLOR;

/* cld_console is similar in format to printf, and also natively supports
    setting a foreground and background color.

   For example the following will output 'The quick brown fox' on a black background with green text:
       cld_console(CLD_CONSOLE_GREEN, CLD_CONSOLE_BLACK, "The quick brown %s\n\r", "fox");

    cld_console returns CLD_SUCCESS if the requested message has been accepted
    and will be transmit using the console UART. CLD_FAIL is returned if the
    requested message will not be transmitted.
*/
extern CLD_RV cld_console(CLD_CONSOLE_COLOR foreground_color, CLD_CONSOLE_COLOR background_color, const char *fmt, ...);

#ifdef __cplusplus
}
#endif

#endif /* _LANGUAGE_C */

#endif  /* __CLD_AUDIO_1_0_LIB__ */
