/*==============================================================================
    FILE:           main.c

    DESCRIPTION:

    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/

#include "user_audio.h"
#include "user_adau1761.h"
/* ADI initialization header */
#include "adi_initialize.h"
#include <stdio.h>

#define LED1_PORT (ADI_GPIO_PORT_A)
#define LED1_PIN  (ADI_GPIO_PIN_0)

#define LED2_PORT (ADI_GPIO_PORT_A)
#define LED2_PIN  (ADI_GPIO_PIN_1)

#define LED3_PORT (ADI_GPIO_PORT_B)
#define LED3_PIN  (ADI_GPIO_PIN_1)
#define GPIO_MEMORY_SIZE (ADI_GPIO_CALLBACK_MEM_SIZE*2)

static unsigned char gpioMemory[GPIO_MEMORY_SIZE];

typedef enum
{
    MAIN_STATE_SYSTEM_INIT,
    MAIN_STATE_USER_INIT_CODEC,
    MAIN_STATE_USER_INIT,
    MAIN_STATE_RUN,
    MAIN_STATE_ERROR
} Main_States;


extern void ConfigSoftSwitches(void);

void configGpio(void)
{
    int i;
    uint32_t gpioMaxCallbacks;
    ADI_INT_STATUS  eIntResult;

    // init the GPIO service
    ADI_GPIO_RESULT result = adi_gpio_Init(
            (void*)gpioMemory,
            GPIO_MEMORY_SIZE,
            &gpioMaxCallbacks);

    /*
     * Setup LED's
     */

    // set GPIO output LED
    result = adi_gpio_SetDirection(
        LED1_PORT,
        LED1_PIN,
        ADI_GPIO_DIRECTION_OUTPUT);

    // set GPIO output LED
    result = adi_gpio_SetDirection(
        LED2_PORT,
        LED2_PIN,
        ADI_GPIO_DIRECTION_OUTPUT);

    // set GPIO output LED
    result = adi_gpio_SetDirection(
        LED3_PORT,
        LED3_PIN,
        ADI_GPIO_DIRECTION_OUTPUT);
}


int main(void)
{
    Main_States main_state = MAIN_STATE_SYSTEM_INIT;
    User_Audio_Init_Return_Code rv;

    while (1)
    {
        switch (main_state)
        {
            case MAIN_STATE_SYSTEM_INIT:
                adi_initComponents(); /* auto-generated code */

                /* Software Switch Configuration for the EZ-BOARD */
                ConfigSoftSwitches();

                configGpio();

                main_state = MAIN_STATE_USER_INIT_CODEC;
            break;
            case MAIN_STATE_USER_INIT_CODEC:
                if (user_adau1761_init() == USER_ADAU1761_INIT_SUCCESS)
                {
                    main_state = MAIN_STATE_USER_INIT;
                }
                else
                {
                    main_state = MAIN_STATE_ERROR;
                }
            break;
            case MAIN_STATE_USER_INIT:
                rv = user_audio_init();
                if (rv == USER_AUDIO_INIT_SUCCESS)
                {
                    main_state = MAIN_STATE_RUN;
                }
                else if (rv == USER_AUDIO_INIT_FAILED)
                {
                    main_state = MAIN_STATE_ERROR;
                }
            break;

            case MAIN_STATE_RUN:
                 user_audio_main();
                 user_adau1761_main();
            break;

            case MAIN_STATE_ERROR:

            break;
        }
    }
}
