#ifndef __USER_ADAU1761__
#define __USER_ADAU1761__
/*==============================================================================
    FILE:           user_adau1761.h

    DESCRIPTION:    Provides a interface between the user_audio USB Audio 1.0
                    functionality and the ADI adau1761 driver.

    Copyright (c) 2015 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to reproduce and distribute executable files
    created using this software for use on Analog Devices Blackfin family
    processors only. Nothing else gives you the right to use this software.

==============================================================================*/

#include "codec/adi_adau1761.h"
//#include <adi_adau1761.h>

/* The SPORT device selection depends on which BF707 EZ-Board connector
 * the Audio EZ-Extender is attached.
 * P1A - Sport TX 0, Sport RX 0
 * P1B - Sport TX 1, Sport RX 1
 */
#define SPORT_TX_DEVICE  0
#define SPORT_RX_DEVICE  0

/* TWI device instance used for communicating with the codec device */
#define TWI_DEV_NUM             0

typedef enum
{
    USER_ADAU1761_INIT_SUCCESS = 0,
    USER_ADAU1761_INIT_ONGOING,
    USER_ADAU1761_INIT_FAILED,
} User_ADAU1761_Init_Return_Code;

/* adau1761 device handle */
extern ADI_ADAU1761_HANDLE  hADAU1761;

/* Structure used to send received USB audio data to the ADAU1761 Codec */
typedef struct
{
    unsigned short buffer_size;             /* Number of bytes being transferred */
    unsigned char * p_buffer;               /* Pointer to the data being transfered */
    void (*fp_xfr_complete) (void);         /* Function called once the data is sent
                                               to the codec. */
} User_ADAU1761_Audio_Transfer_Params;

extern User_ADAU1761_Init_Return_Code user_adau1761_init (void);
extern void user_adau1761_main (void);
extern ADI_ADAU1761_RESULT user_adau1761_SetVolume (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel,
    unsigned short                 const usb_volume);
extern ADI_ADAU1761_RESULT user_adau1761_SetMute (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    unsigned char                  const mute);

extern ADI_ADAU1761_RESULT user_adau1761_GetVolume (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel,
    unsigned short *                p_volume);

extern unsigned char user_adau1761_GetMute (
    ADI_ADAU1761_HANDLE            const hDevice,
    ADI_ADAU1761_VOLUME_CONTROL    const eVolumeCtrl,
    ADI_ADAU1761_VOLUME_CHANNEL    const eChannel);
extern unsigned short user_adau1761_convert_1761_volume_to_usb (unsigned char adau1761_vol);
unsigned short user_adau1761_convert_mic_1761_volume_to_usb (unsigned char adau1761_vol);
extern unsigned char user_adau1761_convert_usb_volume_to_codec (unsigned short usb_vol);
extern unsigned char user_adau1761_convert_usb_volume_to_codec_mic (unsigned short usb_vol);
extern void user_adau1761_transfer_audio_buffer (User_ADAU1761_Audio_Transfer_Params * p_params);
extern void user_adau1761_enable_output (void);
extern void user_adau1761_disable_output (void);
extern void user_adau1761_enable_input (void);
extern void user_adau1761_disable_input (void);
#endif /* __USER_ADAU1761__ */
